package Thread is
    
    pragma Assertion_Policy(Pre => Check);
    pragma Assertion_Policy(Post => Check);

    type State is (None, Ready, Running, Stopped, Sleeping, Waiting);
    type Action is (Notify, Resume, Sleep, Start, Stop, Wait);

    procedure Initialize(S: out State) with
        Post => (S = Ready);
    -- Sets S to Ready.

    function Is_Valid_State(S: State) return Boolean is
        (S /= None);

    procedure Do_Action(S: in out State; A: Action) with
        Pre  => Is_Valid_State(S), 
      	Post =>   ((S'Old = Ready)    and then (A = Start)  and then (S = Running))
          or else ((S'Old = Ready)    and then (A = Stop)   and then (S = Stopped))
          or else ((S'Old = Running)  and then (A = Sleep)  and then (S = Sleeping))
          or else ((S'Old = Running)  and then (A = Wait)   and then (S = Waiting))
          or else ((S'Old = Running)  and then (A = Stop)   and then (S = Stopped))
          or else ((S'Old = Sleeping) and then (A = Resume) and then (S = Running))
          or else ((S'Old = Waiting)  and then (A = Notify) and then (S = Running))
          or else (S = None);
    -- Updates the state S according to the given input state S, and the 
    -- given action A. Sets S to None if the transition is not defined.
end Thread;