with AUnit.Assertions;
with System.Assertions;
with Thread;

use AUnit.Assertions;
use System.Assertions;
use Thread;

package body Thread_Tests is
    
    procedure Test_Invalid_Action(S: in out State; A: Action) is
    begin
        Do_Action(S, A);
        Assert(S = None, 
            "Action was expected to be invalid. S should be None.");
    end Test_Invalid_Action;

    -- ---------------------------------------------------------------

    procedure Prepare_Ready(S: out State) is 
    begin
        Initialize(S);
    end Prepare_Ready;

    -- ---------------------------------------------------------------

    procedure Prepare_Running(S: out State) is
    begin
        Initialize(S);
        Do_Action(S, Start);
    end Prepare_Running;

    -- ---------------------------------------------------------------

    procedure Prepare_Sleeping(S: out State) is
    begin
        Initialize(S);
        Do_Action(S, Start);
        Do_Action(S, Sleep);
    end Prepare_Sleeping;

    -- ---------------------------------------------------------------

    procedure Prepare_Stopped(S: out State) is
    begin
        Initialize(S);
        Do_Action(S, Stop);
    end Prepare_Stopped;

    -- ---------------------------------------------------------------

    procedure Prepare_Waiting(S: out State) is
    begin
        Initialize(S);
        Do_Action(S, Start);
        Do_Action(S, Wait);
    end Prepare_Waiting;

    -- ---------------------------------------------------------------

    procedure Test_Failing_Action(S: in out State; A: Action) is
    begin
        Do_Action(S, A);
        Assert(False, "Exception expected");
    exception
        when Assert_Failure => return;
        when others => Assert(False, "Incorrect exception");
    end Test_Failing_Action;

    -- ---------------------------------------------------------------

    procedure Test_None_Invalid(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State := None;
    begin
        Test_Failing_Action(S, Start);

        S := None;
        Test_Failing_Action(S, Notify);
        
        S := None;
        Test_Failing_Action(S, Resume);
        
        S := None;
        Test_Failing_Action(S, Sleep);
        
        S := None;
        Test_Failing_Action(S, Wait);
        
        S := None;
        Test_Failing_Action(S, Stop);
    end Test_None_Invalid;

    -- ---------------------------------------------------------------

    procedure Test_Initialize(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State := None;
    begin
        Prepare_Ready(S);
        Assert(S = Ready, "Test_Initialize failed.");
    end Test_Initialize;
    
    -- ---------------------------------------------------------------

    procedure Test_Initialize_Invalid(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State := None;
    begin
        Prepare_Ready(S);
        Test_Invalid_Action(S, Notify);
        
        Prepare_Ready(S);
        Test_Invalid_Action(S, Resume);
        
        Prepare_Ready(S);
        Test_Invalid_Action(S, Sleep);
        
        Prepare_Ready(S);
        Test_Invalid_Action(S, Wait);
    end Test_Initialize_Invalid;
    
    -- ---------------------------------------------------------------

    procedure Test_Ready(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Initialize(S);
        Do_Action(S, Start);
        Assert(S = Running, "Test_Ready failed.");

        Do_Action(S, Stop);
        Assert(S = Stopped, "Test_Ready failed.");
    end Test_Ready;
    
    -- ---------------------------------------------------------------

    procedure Test_Ready_Invalid(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Prepare_Ready(S);
        Test_Invalid_Action(S, Notify);

        Prepare_Ready(S);
        Test_Invalid_Action(S, Resume);
        
        Prepare_Ready(S);
        Test_Invalid_Action(S, Sleep);
        
        Prepare_Ready(S);
        Test_Invalid_Action(S, Wait);
    end Test_Ready_Invalid;
    
    -- ---------------------------------------------------------------

    procedure Test_Running(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Prepare_Running(S);
        Do_Action(S, Wait);
        Assert(S = Waiting, "Test_Running 1 failed.");

        Prepare_Running(S);
        Do_Action(S, Sleep);
        Assert(S = Sleeping, "Test_Running 2 failed.");

        Prepare_Running(S);
        Do_Action(S, Stop);
        Assert(S = Stopped, "Test_Running 3 failed.");
    end Test_Running;
    
    -- ---------------------------------------------------------------

    procedure Test_Running_Invalid(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Prepare_Running(S);
        Test_Invalid_Action(S, Notify);

        Prepare_Running(S);
        Test_Invalid_Action(S, Resume);

        Prepare_Running(S);
        Test_Invalid_Action(S, Start);
    end Test_Running_Invalid;
    
    -- ---------------------------------------------------------------

    procedure Test_Sleeping(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Prepare_Sleeping(S);
        Do_Action(S, Resume);
        Assert(S = Running, "Test_Sleeping failed.");
    end Test_Sleeping;
    
    -- ---------------------------------------------------------------

    procedure Test_Sleeping_Invalid(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Prepare_Sleeping(S);
        Test_Invalid_Action(S, Sleep);

        Prepare_Sleeping(S);
        Test_Invalid_Action(S, Start);

        Prepare_Sleeping(S);
        Test_Invalid_Action(S, Stop);

        Prepare_Sleeping(S);
        Test_Invalid_Action(S, Wait);
    end Test_Sleeping_Invalid;
    
    -- ---------------------------------------------------------------

    procedure Test_Stopped(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Prepare_Stopped(S);
        Assert(S = Stopped, "Test_Stopped failed.");
    end Test_Stopped;
    
    -- ---------------------------------------------------------------

    procedure Test_Stopped_Invalid(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Prepare_Stopped(S);
        Test_Invalid_Action(S, Notify);

        Prepare_Stopped(S);
        Test_Invalid_Action(S, Resume);

        Prepare_Stopped(S);
        Test_Invalid_Action(S, Sleep);

        Prepare_Stopped(S);
        Test_Invalid_Action(S, Start);

        Prepare_Stopped(S);
        Test_Invalid_Action(S, Stop);

        Prepare_Stopped(S);
        Test_Invalid_Action(S, Wait);
    end Test_Stopped_Invalid;
    
    -- ---------------------------------------------------------------

    procedure Test_Waiting(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Prepare_Waiting(S);
        Do_Action(S, Notify);
        Assert(S = Running, "Test_Waiting failed.");
    end Test_Waiting;
    
    -- ---------------------------------------------------------------

    procedure Test_Waiting_Invalid(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        S: State;
    begin
        Prepare_Waiting(S);
        Test_Invalid_Action(S, Resume);

        Prepare_Waiting(S);
        Test_Invalid_Action(S, Sleep);

        Prepare_Waiting(S);
        Test_Invalid_Action(S, Start);

        Prepare_Waiting(S);
        Test_Invalid_Action(S, Stop);

        Prepare_Waiting(S);
        Test_Invalid_Action(S, Wait);
    end Test_Waiting_Invalid;
    
    -- ---------------------------------------------------------------

    function Name(T: Test) return Test_String is
        pragma Unreferenced(T);
    begin
        return Format("Thread_Tests");
    end Name;
    
    -- ---------------------------------------------------------------

    procedure Register_Tests(T: in out Test) is
        use AUnit.Test_Cases.Registration;
    begin
        Register_Routine(T, Test_None_Invalid'Access, "Test_None_Invalid");
        Register_Routine(T, Test_Initialize'Access, "Test_Initialize");
        Register_Routine(T, Test_Initialize_Invalid'Access, 
            "Test_Initialize_Invalid");
        Register_Routine(T, Test_Ready'Access, "Test_Ready");
        Register_Routine(T, Test_Ready_Invalid'Access, 
            "Test_Ready_Invalid");
        Register_Routine(T, Test_Waiting'Access, "Test_Waiting");
        Register_Routine(T, Test_Waiting_Invalid'Access, 
            "Test_Waiting_Invalid");
        Register_Routine(T, Test_Running'Access, "Test_Running");
        Register_Routine(T, Test_Running_Invalid'Access, 
            "Test_Running_Invalid");
        Register_Routine(T, Test_Sleeping'Access, "Test_Sleeping");
        Register_Routine(T, Test_Sleeping_Invalid'Access, 
            "Test_Sleeping_Invalid");
        Register_Routine(T, Test_Stopped'Access, "Test_Stopped");
        Register_Routine(T, Test_Stopped_Invalid'Access, 
            "Test_Stopped_Invalid");
    end Register_Tests;
    
end Thread_Tests;
