Context
	with Ada.Text_IO; use Ada.Text_IO;
	with Ada.Numerics.Elementary_Functions; use Ada.Numerics.Elementary_Functions;
	with Vectors; use Vectors;

***** Test Add
Define	L: constant Vector := (2.0, 3.0, 4.0);
		R: constant Vector := (5.0, 6.0, 7.0);
		Actual: Vector;
		Expected: constant Vector := (L.X + R.X, L.Y + R.Y, L.Z + R.Z);
Test	Actual := L + R;
Pass	Actual = Expected

***** Test_Are_Equal
Define	L: constant Vector := (2.0, 3.0, 4.0);
		R: constant Vector := (2.0, 3.0, 4.0);
		Actual: Boolean;
Test	Actual := L = R;
Pass	Actual

***** Test_Are_Not_Equal
Define	V: constant Vector := (2.0, 3.0, 4.0);
		X_False: constant Vector := (7.0, 3.0, 4.0);
		Y_False: constant Vector := (2.0, -3.0, 4.0);
		Z_False: constant Vector := (2.0, 3.0, -4.0);
		Are_V_And_X_Equal, Are_V_And_Y_Equal, Are_V_And_Z_Equal: Boolean;
Test	Are_V_And_X_Equal := V = X_False;
		Are_V_And_Y_Equal := V = Y_False;
		Are_V_And_Z_Equal := V = Z_False;
Pass	(not Are_V_And_X_Equal) 
			And (not Are_V_And_Y_Equal) 
			And (not Are_V_And_Z_Equal)

***** Test_Are_Orthogonal
Define 	L: constant Vector := (2.0, 3.0, -4.0);
	    R: constant Vector := (5.0, 6.0, 7.0);
		Actual: Boolean;
Test	Actual := Are_Orthogonal(L, R);
Pass	Actual
	
***** Test_Are_Not_Orthogonal
Define 	L: constant Vector := (2.0, 3.0, 4.0);
	    R: constant Vector := (5.0, 6.0, 7.0);
		Actual: Boolean;
Test	Actual := Are_Orthogonal(L, R);
Pass	not Actual
	
***** Test_Cross_Product
Define	L: constant Vector := (2.0, 3.0, 4.0);
	    R: constant Vector := (5.0, 6.0, 7.0);
	    Expected: constant Vector := (L.Y * R.Z - L.Z * R.Y, L.Z * R.X - L.X * R.Z, L.X * R.Y - L.Y * R.X);
		Actual: Vector;
Test	Actual := Cross_Product(L, R);
Pass	Actual = Expected

***** Test_Distance
Define	L: constant Vector := (2.0, 3.0, 4.0);
	    R: constant Vector := (5.0, 6.0, 7.0);
	    Expected: constant Float := Sqrt((L.X - R.X)**2 + (L.Y - R.Y)**2 + (L.Z - R.Z)**2);
	    Actual: Float;
Test	Actual := Distance(L, R);
Pass	Actual = Expected

***** Test_Distance_To_Origin
Define	Item: constant Vector := (2.0, 3.0, 4.0);
	    Actual: Float;
	    Expected: constant Float := Sqrt(Item.X**2 + Item.Y**2 + Item.Z**2);
Test	Actual := Distance_To_Origin(Item);
Pass	Actual = Expected
    
***** Test_Subtract
Define	L: constant Vector := (2.0, 3.0, 4.0);
	    R: constant Vector := (5.0, 6.0, 7.0);
	    Expected: constant Vector := (L.X - R.X, L.Y - R.Y, L.Z - R.Z);
	    Actual: Vector;
Test	Actual := L - R;
Pass	Actual = Expected
	
***** Test_Scalar
Define	L: constant Vector := (2.0, 3.0, 4.0);
	    R: constant Vector := (5.0, 6.0, 7.0);
	    Actual: Float;
	    Expected: constant Float := L.X * R.X + L.Y * R.Y + L.Z * R.Z;
Test	Actual := L * R;
Pass	Actual = Expected
