package body Binary_Trees is

    function Internal_Create(Item: Item_Type; Parent: Binary_Tree_Access) 
        return Binary_Tree_Access is
    begin
        return new Binary_Tree'(
             Item    => Item, 
             Left    => null, 
             Right   => null, 
             Parent  => Parent
        );
    end Internal_Create;

    -- ---------------------------------------------------------------

    procedure Internal_Add_Item(T: in out Binary_Tree_Access; 
                                Parent: Binary_Tree_Access; 
                                Item: Item_Type) is
    begin
        if T = null then
            T := Internal_Create(Item, Parent);
        elsif Item = T.Item then
            raise Item_Already_In_Tree_Exception;
        elsif Item < T.Item then
            Internal_Add_Item(T.Left, T, Item);
        else
            Internal_Add_Item(T.Right, T, Item);
        end if;
    end Internal_Add_Item;

    -- ---------------------------------------------------------------

    procedure Add_Item(T : in out Binary_Tree_Access; Item: Item_Type) is
    begin
        Internal_Add_Item(T, T, Item);
    end Add_Item;

    -- ---------------------------------------------------------------

    function Create(Item: Item_Type) return Binary_Tree_Access is
    begin
        return Internal_Create(Item, null);
    end Create;

    -- ---------------------------------------------------------------

    function Has_Children(T: Binary_Tree_Access) return Boolean is
        Children_Check : Boolean := True;
    begin
        if T.Right = null and T.Left = null then
            Children_Check := False;
        end if;
        return Children_Check;
    end Has_Children;

    -- ---------------------------------------------------------------

    function Has_Item(T: Binary_Tree_Access; Item: Item_Type) return Boolean is
    begin
        if T = null then
            return False;
        elsif Item = T.Item then
            return True;
        elsif Item < T.Item then
            return Has_Item(T.Left, Item);
        else
            return Has_Item(T.Right, Item);
        end if;
    end Has_Item;

    -- ---------------------------------------------------------------

    function Get_Height(T: Binary_Tree_Access) return Natural is
    begin
        if T = null then
            return 0;
        else
            return Integer'Max(Get_Height(T.Left), Get_Height(T.Right)) + 1;
        end if;
    end Get_Height;

    -- ---------------------------------------------------------------

    function Get_Num_Leaves(T: Binary_Tree_Access) return Natural is
    begin
        if T = null then
            return 0;
        else
            return Get_Num_Leaves(T.Left) + Get_Num_Leaves(T.Right) + 1;
        end if;
    end Get_Num_Leaves;

    -- ---------------------------------------------------------------

    procedure Put(T: Binary_Tree_Access) is
    begin
        if T = null then
            return;
        else
            Put(T.Left);
            Put_Item(T.Item);
            Put(T.Right);
        end if;
    end Put;

    -- ---------------------------------------------------------------

    procedure Find_And_Unlink_Max(T: in out Binary_Tree_Access;
                                  Max: out Item_Type) is
        Left: Binary_Tree_Access;
    begin
        if T.Right = null then
            Max := T.Item;
            Left := T.Left;
            Free(T);
            T := Left;
        else
            Find_And_Unlink_Max(T.Right, Max);
        end if;
    end Find_And_Unlink_Max;

    -- ---------------------------------------------------------------

    procedure Delete_Root(T: in out Binary_Tree_Access) is
        To_Recycle : Binary_Tree_Access;
    begin
        if T.Left = null and T.Right = null then
            Free(T);
        elsif T.Left = null then
            To_Recycle := T;
            T := T.Right;
            Free(To_Recycle);
        elsif T.Right = null then
            To_Recycle := T;
            T := T.Left;
            Free(To_Recycle);
        else
            Find_And_Unlink_Max(T, T.Item);
        end if;
    end Delete_Root;

    -- ---------------------------------------------------------------

    procedure Remove_Item(T: in out Binary_Tree_Access; Item: Item_Type) is
    begin
        if T = null then
            raise Item_Not_Found_Exception;
        elsif T.Item = Item then
            Delete_Root(T);
        elsif Item < T.Item then
            Remove_Item(T.Left, Item);
        else
            Remove_Item(T.Right, Item);
        end if;
    end Remove_Item;

    -- ---------------------------------------------------------------

    procedure Remove_All(T: in out Binary_Tree_Access) is
    begin
        if T = null then
            return;
        elsif T.Left = null and T.Right = null then
            Free(T);
        else
            Remove_All(T.Left);
            Remove_All(T.Right);
            Free(T);
        end if;
    end Remove_All;

end Binary_Trees;
